#include <inttypes.h>
#include <i2c.h>

extern void Delay_ms(uint32_t ms);

#define LCD_CMD  0b00000000
#define LCD_DATA 0b00000001
#define E_HIGH   0b00000100
#define E_LOW    0b11111011
#define BL_ON    0b00001000
#define BL_OFF   0b11110111

// global Variables

char const addLUT[2] = {0x80,0xC0};	// RAM Offset for 2x16 LCD only
uint8_t _lcd_addr;
uint8_t _lcd_line;
uint8_t _lcd_bgl;


/******************************************************************************
* Function:     lcd_bgl(uint8_t data)
* Description:  switch LCD Background Light on/off at next output
* Returns:      none
*******************************************************************************/
void lcd_bgl(uint8_t backgroundlight)
{
	if(backgroundlight)
		_lcd_bgl=BL_ON;
	else
		_lcd_bgl=0;
}
/******************************************************************************
* Function:     lcd_wr_half(uint8_t data)
* Description:  Write a nibble to the LCD display using DB4-DB7
* Returns:      none
*******************************************************************************/
void lcd_wr_half(uint8_t data)
{
    data<<=4;
    data|=_lcd_bgl;
    pcf8574_write(0,PCF8574_HW_ADDRESS,data|E_HIGH);
    pcf8574_write(0,PCF8574_HW_ADDRESS,data&E_LOW);
    Delay_ms(3);    // allow display time to accept the new data
}

/******************************************************************************
* Function:     lcd_wr_disp(uint8_t data)
* Description:  Write a byte to the LCD display
* Returns:      none
*******************************************************************************/
void lcd_wr_disp(uint8_t data, int cmd_mode)
{
    uint8_t temp;

    // Write to display upper nibble
    temp = data &0xf0;
    temp |=  cmd_mode|_lcd_bgl;
    pcf8574_write(0,PCF8574_HW_ADDRESS,temp | E_HIGH);
    pcf8574_write(0,PCF8574_HW_ADDRESS,temp & E_LOW);

    // Write to display lower nibble
    temp = (data & 0x0f)<<4;
    temp |=  cmd_mode|_lcd_bgl;
    pcf8574_write(0,PCF8574_HW_ADDRESS,temp | E_HIGH);
    pcf8574_write(0,PCF8574_HW_ADDRESS,temp & E_LOW);
}



/******************************************************************************
* Function:     lcd_init(void)
* Description:  Setup LCD display for application
* Returns:      none
*******************************************************************************/
void lcd_init_disp(void)
{
    // always command mode used here
    lcd_wr_half(0x03);          // Send 0x33 and 0x32 to enter 4 bit mode
    lcd_wr_half(0x03);          // See HD44780U.PDF datasheet for all
    lcd_wr_half(0x03);          // configuration options
    lcd_wr_half(0x02);

    lcd_wr_disp(0x28, LCD_CMD);  // Mode: 4-Bit, 2-line, 5x7
    lcd_wr_disp(0x01, LCD_CMD);  // Clear Display memory
    Delay_ms(20);
    lcd_wr_disp(0x10, LCD_CMD);  // Set cursor to move
    lcd_wr_disp(0x06, LCD_CMD);  // set cursor to move right, no shifting of LCD display
    lcd_wr_disp(0x0c, LCD_CMD);  // turn display on, cursor off, cursor blinking off
}

/******************************************************************************
* Function:     void line(char which_line)
* Description:  Move to line specified by which_line
* Uses:         which_line to selct LCD line to be used
* Returns:      none
*******************************************************************************/
void lcd_line(uint8_t which_line)
{
    _lcd_addr = addLUT[which_line - 1];
    lcd_wr_disp(_lcd_addr, LCD_CMD);                     // move to which_line line
    _lcd_line = which_line;
}

/******************************************************************************
* Function:     void lcd_clear(void)
* Description:  Clear LCD
* Uses:         none
* Returns:      none
*******************************************************************************/
void lcd_clear(void)
{
    lcd_wr_disp(0x01, LCD_CMD);              // Clear display and home cursor
    Delay_ms(20);
    lcd_line(1);
}

/******************************************************************************
* Function:     void lcd_set_cur(uint8_t LCDrow, uint8_t LCDcol)
* Description:  Set row, col cursor position in LCD
* Uses:         LCDrow, LCDcol
* Returns:      none
*******************************************************************************/
void lcd_set_cur(uint8_t LCDrow, uint8_t LCDcol)
{
    _lcd_addr = addLUT[LCDrow] + LCDcol;
    lcd_wr_disp(_lcd_addr, LCD_CMD);          // move to row and column
    _lcd_line = LCDrow;
}

/******************************************************************************
* Function:     void lcd_put(uint8_t c)
* Description:  display a character on LCD
* Uses:         character c, gl _lcd_line, _lcd_addr
* Returns:      none
*******************************************************************************/
void lcd_put(uint8_t  c)
{
    lcd_wr_disp(c, LCD_DATA);
    _lcd_addr++;

    if(_lcd_addr == 16)
        lcd_line(2);

    if(_lcd_addr == 32)
        lcd_line(1);
}
/******************************************************************************
* Function:     void lcd_puts(char * s)
* Description:  display a string on LCD
* Uses:         character c, gl _lcd_line, _lcd_addr
* Returns:      none
*******************************************************************************/
void lcd_puts(char * s)
{
	while(*s)
		lcd_put((uint8_t)*s++);
}

