/* serielle Kommunikation fuer STM32F103C8T6 */

#include <inttypes.h>
#include "stm32_lutz.h"
#include "stm32f10x_usart.h"
#include "stm32f10x_conf.h"
#include "misc.h"
#include "usart.h"

NVIC_InitTypeDef NVIC_InitStructure;
USART_InitTypeDef USART_InitStructure;




/* Funktionsprinzip:
Der eigentliche Datenverkehr ueber die diversen UART's wird
weitgehend interruptgesteuert erledigt. Als Schnittstelle
zu den Anwendungen dienen Sende- und Empfangspuffer, die mit
entsprechenden Funktionen abgefragt und befuellt werden
koennen.
 */

/* Pufferlaengen: immer eine 2er Potenz */
#define OBLEN  32
#define IBLEN  8

#define TUBUF  struct T_UARTBUF
struct T_UARTBUF
{ char OutBuf[OBLEN];
  char InBuf[IBLEN];
  volatile int  InWP;
  volatile int  InRP;
  volatile int  OutWP;
  volatile int  OutRP;
};

/* die Puffer fuer alle 3 USART */
TUBUF U1Buf;
TUBUF U2Buf;
TUBUF U3Buf;


TUBUF* GetUsartBuffer (int uart)
{ switch(uart)
  { case 1: return &U1Buf;
    case 2: return &U2Buf;
    case 3: return &U3Buf;
  }
  return 0;
}

/*******************************************************************************
* Function Name  : NVIC_SetIRQChannelPendingBit
* Description    : Sets the NVICs interrupt pending bit.
* Input          : - NVIC_IRQChannel: specifies the interrupt pending bit to Set.
* Output         : None
* Return         : None
*******************************************************************************/
void NVIC_SetIRQChannelPendingBit(u8 NVIC_IRQChannel)
{
  /* Check the parameters */
  assert_param(IS_NVIC_IRQ_CHANNEL(NVIC_IRQChannel));
  
  *(u32*)0xE000EF00 = (u32)NVIC_IRQChannel;
}

/************** Interrupt-Handler ****************/
/*
merke:
die Interrupt-Bits in USARTx_ISR sind statisch!
also erzeugen sie SOFORT einen neuen Interrupt,
wenn man sie nach dem letzten gesendeten Byte
nicht per USARTx_CR1 ausschaltet.
*/

void USART1_IRQHandler (void)
{ char c;
  int  i, j;

  if (USART1->SR & ((1<<5)|(1<<3)))     // RX: Zeichen empfangen oder Overrun
  { c = USART1->DR;
    i = U1Buf.InWP;
    j = (i+1) & (IBLEN-1);
    if (j!=U1Buf.InRP)
    { U1Buf.InBuf[i] = c;
      U1Buf.InWP = j;
    }
  }

  if  (USART1->SR & (1<<7))              // TX: Sendepuffer leer geworden
  { i = U1Buf.OutRP;
    if (i!=U1Buf.OutWP)                  // ob es was zu senden gibt
    { USART1->DR  = U1Buf.OutBuf[i];
      U1Buf.OutRP = (i+1) & (OBLEN-1);
    }
    else
    { USART1->CR1 &= ~(3<<6);             // noe, TXEIE und TCIE ausschalten
    }
  }
}

void USART2_IRQHandler (void)
{ char c;
  int  i, j;

  if (USART2->SR & ((1<<5)|(1<<3)))      // RX: Zeichen empfangen
  { c = USART2->DR;
    i = U2Buf.InWP;
    j = (i+1) & (IBLEN-1);
    if (j!=U2Buf.InRP)
    { U2Buf.InBuf[i] = c;
      U2Buf.InWP = j;
    }
  }

  if  (USART2->SR & (1<<7))              // TX: Sendepuffer leer?
  { i = U2Buf.OutRP;
    if (i!=U2Buf.OutWP)                  // ob es was zu senden gibt
    { USART2->DR  = U2Buf.OutBuf[i];
      U2Buf.OutRP = (i+1) & (OBLEN-1);
    }
    else
    { USART2->CR1 &= ~(3<<6);             // n, TXEIE und TCIE ausschalten
    }
  }
}

void USART3_IRQHandler (void)
{ char c;
  int  i, j;

  if (USART3->SR & ((1<<5)|(1<<3)))      // RX: Zeichen empfangen
  { c = USART3->DR;
    i = U3Buf.InWP;
    j = (i+1) & (IBLEN-1);
    if (j!=U3Buf.InRP)
    { U3Buf.InBuf[i] = c;
      U3Buf.InWP = j;
    }
  }

  if  (USART3->SR & (1<<7))              // TX: Sendepuffer leer?
  { i = U3Buf.OutRP;
    if (i!=U3Buf.OutWP)                  // ob es was zu senden gibt
    { USART3->DR  = U3Buf.OutBuf[i];
      U3Buf.OutRP = (i+1) & (OBLEN-1);
    }
    else
    { USART3->CR1 &= ~(3<<6);             // n, TXEIE und TCIE ausschalten
    }
  }
}


/************* USART/UART aufsetzen **************************/

/*
Baudrate errechnen:
Die Faustformel ist:  Baudrate = Clock / BRR;
Clock ist hier 72 MHz, da SysClock als Taktquelle gewhlt.

BRR ist eigentlich eine 16 Bit Festkommazahl:
die untersten 4 Bit sind der gebrochene Anteil
Formel lt. Manual   Baud = 36MHz / (16*BRR);
aber da im BRR Register die BRR ja ohnehin mit 4 Nachkommabits steht,
ist sie bereits an der richtigen Stelle und
    BRR_value = 36MHz / Baudrate;
ist bereits die richtige Formel.

 Voraussetzung: U(S)ART muss in RCC_APBxENR bereits freigegeben sein
 Return: tatsaechliche Baudrate
 */

void Init_USART1 ()
{

  U1Buf.InWP =                      /* alle Puffer ruecksetzen */
  U1Buf.InRP =
  U1Buf.OutWP =
  U1Buf.OutRP = 0;
  /* Int 37 */


  NVIC_SetIRQChannelPendingBit(LUTZ_COM1_IRQn);

  USART_DeInit(USART1);
  USART_InitStructure.USART_BaudRate = 1200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;  // Word Length = 8 Bits
  USART_InitStructure.USART_StopBits = USART_StopBits_1;  // One Stop Bit
  USART_InitStructure.USART_Parity = USART_Parity_No ;   // No parity
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; 
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  //USART_InitStructure.USART_Mode =  USART_Mode_Tx;

  USART_Init(USART1,&USART_InitStructure);
  USART_ITConfig(USART1, USART_IT_RXNE, DISABLE);
  USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
  USART_Cmd(USART1, ENABLE);


  NVIC_InitStructure.NVIC_IRQChannel = LUTZ_COM1_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}

void Init_USART2 ()
{

  U2Buf.InWP =                      /* alle Puffer ruecksetzen */
  U2Buf.InRP =
  U2Buf.OutWP =
  U2Buf.OutRP = 0;
  /* Int 38 */

  NVIC_SetIRQChannelPendingBit(LUTZ_COM2_IRQn);


  USART_DeInit(USART2);
  USART_InitStructure.USART_BaudRate = 1200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;  // Word Length = 8 Bits
  USART_InitStructure.USART_StopBits = USART_StopBits_1;  // One Stop Bit
  USART_InitStructure.USART_Parity = USART_Parity_No ;   // No parity
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; 
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  //USART_InitStructure.USART_Mode =  USART_Mode_Tx;

  USART_Init(USART1,&USART_InitStructure);
  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
  USART_Cmd(USART2, ENABLE);

  NVIC_InitStructure.NVIC_IRQChannel = LUTZ_COM2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}

void Init_USART3 ()
{

  U3Buf.InWP =                      /* alle Puffer ruecksetzen */
  U3Buf.InRP =
  U3Buf.OutWP =
  U3Buf.OutRP = 0;
  /* Int 39 */

  NVIC_SetIRQChannelPendingBit(LUTZ_COM3_IRQn);

  USART_DeInit(USART3);
  USART_InitStructure.USART_BaudRate = 38400;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;  // Word Length = 8 Bits
  USART_InitStructure.USART_StopBits = USART_StopBits_1;  // One Stop Bit
  USART_InitStructure.USART_Parity = USART_Parity_No ;   // No parity
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; 
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  //USART_InitStructure.USART_Mode =  USART_Mode_Tx;

  USART_Init(USART3,&USART_InitStructure);
  USART_ITConfig(USART3, USART_IT_RXNE, ENABLE);
  USART_Cmd(USART3, ENABLE);

  NVIC_InitStructure.NVIC_IRQChannel = LUTZ_COM3_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}


/****************** I/O Funktionen *******************************/

/* ob empfangene Zeichen vorhanden sind */
int8_t USART1_RxAvail (void) { if (U1Buf.InRP != U1Buf.InWP) return 1;  return 0; }
int8_t USART2_RxAvail (void) { if (U2Buf.InRP != U2Buf.InWP) return 1;  return 0; }
int8_t USART3_RxAvail (void) { if (U3Buf.InRP != U3Buf.InWP) return 1;  return 0; }


/* empfangene Zeichen abholen */
uint8_t USART1_GetChar (void)
{ int32_t i;
  char c;
  i = U1Buf.InRP;  if (i == U1Buf.InWP) return 0;
  c = U1Buf.InBuf[i];  U1Buf.InRP = (i+1) & (IBLEN-1);
  return c;
}

uint8_t USART2_GetChar (void)
{ int32_t i;
  char c;
  i = U2Buf.InRP;  if (i == U2Buf.InWP) return 0;
  c = U2Buf.InBuf[i];  U2Buf.InRP = (i+1) & (IBLEN-1);
  return c;
}

uint8_t USART3_GetChar (void)
{ int32_t i;
  char c;
  i = U3Buf.InRP;  if (i == U3Buf.InWP) return 0;
  c = U3Buf.InBuf[i];  U3Buf.InRP = (i+1) & (IBLEN-1);
  return c;
}


/* Anzahl freier Plaetze im Sendepuffer ermitteln */
int32_t USART1_TxFree (void)
{ int32_t i;
  i = U1Buf.OutWP - U1Buf.OutRP;
  if (i<0) i = i + OBLEN;    /* i = belegte Plaetze */
  return OBLEN - i;
}

int32_t USART2_TxFree (void)
{ int32_t i;
  i = U2Buf.OutWP - U2Buf.OutRP;
  if (i<0) i = i + OBLEN;    /* i = belegte Plaetze */
  return OBLEN - i;
}

int32_t USART3_TxFree (void)
{ int32_t i;
  i = U3Buf.OutWP - U3Buf.OutRP;
  if (i<0) i = i + OBLEN;    /* i = belegte Plaetze */
  return OBLEN - i;
}



/* Zeichen senden */
int8_t USART1_PutChar (int8_t c)
{ int64_t L;
  int32_t i, j;

  i = U1Buf.OutWP;
  j = (i+1) & (OBLEN-1);

  L=100000;
  while (j == U1Buf.OutRP)     // wenn Puffer voll
  { if (L < 0) return 0;    // Timeout
  }

  // Puffer nicht voll, also Zeichen einstapeln
  U1Buf.OutBuf[i] = c;
  U1Buf.OutWP = j;
  USART1->CR1 |= (1<<7);         // Sendeinterrupt einschalten
  return c;
}

int8_t USART2_PutChar (int8_t c)
{ int64_t L;
  int32_t i, j;

  i = U2Buf.OutWP;
  j = (i+1) & (OBLEN-1);

  L=100000;
  while (j == U2Buf.OutRP)     // wenn Puffer voll
  { if (L < 0) return 0;    // Timeout
  }

  // Puffer nicht voll, also Zeichen einstapeln
  U2Buf.OutBuf[i] = c;
  U2Buf.OutWP = j;
  USART2->CR1 |= (1<<7);         // Sendeinterrupt einschalten
  return c;
}

int8_t USART3_PutChar (int8_t c)
{ int64_t L;
  int32_t i, j;

  i = U3Buf.OutWP;
  j = (i+1) & (OBLEN-1);

  L=100000;
  while (j == U3Buf.OutRP)     // wenn Puffer voll
  { if (L < 0) return 0;    // Timeout
  }

  // Puffer nicht voll, also Zeichen einstapeln
  U3Buf.OutBuf[i] = c;
  U3Buf.OutWP = j;
  USART3->CR1 |= (1<<7);         // Sendeinterrupt einschalten
  return c;
}


/* Warten, bis alle Zeichen gesendet sind */
void USART1_TxDone (void) { while (U1Buf.OutRP != U1Buf.OutWP);}
void USART2_TxDone (void) { while (U2Buf.OutRP != U2Buf.OutWP);}
void USART3_TxDone (void) { while (U3Buf.OutRP != U3Buf.OutWP);}


void USART1_PutString(const char *s)
{
    //
    // Loop while there are more characters to send.
    //

	while (*s)
	{
        	USART1_PutChar(*s++);
	}
        // Loop until the end of transmission
        //while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
}

void USART2_PutString(const char *s)
{
    //
    // Loop while there are more characters to send.
    //

	while (*s)
	{
        	USART2_PutChar(*s++);
	}
        // Loop until the end of transmission
        //while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
}

void USART3_PutString(const char *s)
{
    //
    // Loop while there are more characters to send.
    //

	while (*s)
	{
        	USART3_PutChar(*s++);
	}
        // Loop until the end of transmission
        //while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
}


     /**** ende ****/
